import { Monad } from "./fantasy";
import { Optional } from "./optional";
export declare class OptionalAsync<A> implements Monad<A> {
    static of<T>(t: T | null | undefined | Promise<T | null | undefined>): OptionalAsync<T>;
    static empty<T>(): OptionalAsync<T>;
    /**
     * returns a Promise containing an array that keeps only the non empty values of an array of OptionalAsyncs
     * ```
     * const arr = [1, 2, undefined, 4, null, 6].map(OptionalAsync.of);
     * await OptionalAsync.flatten(arr) // => [1, 2, 4, 6]
     * ```
     * @param {Optional<B>[]} arr
     */
    static flatten<T>(arr: OptionalAsync<T>[]): Promise<T[]>;
    static all<T>(optxs: OptionalAsync<T>[]): OptionalAsync<T[]>;
    private readonly pOptA;
    private constructor();
    ap<B>(b: OptionalAsync<(a: A) => B>): OptionalAsync<B>;
    map<B>(fn: (a: A) => B | null | undefined): OptionalAsync<B>;
    mapAsync<B>(fn: (a: A) => Promise<B | null | undefined>): OptionalAsync<B>;
    chain<B>(fn: (a: A) => OptionalAsync<B>): OptionalAsync<B>;
    flatMap<B>(fn: (a: A) => OptionalAsync<B>): OptionalAsync<B>;
    getOrElse(defaultValue: A): Promise<A>;
    get(): Promise<A>;
    getOrThrow(error: Error): Promise<A>;
    getOrNull(): Promise<A | null>;
    getOrUndefined(): Promise<A | undefined>;
    contains(value: A): Promise<boolean>;
    exists(check: (value: A) => boolean): Promise<boolean>;
    promise(): Promise<Optional<A>>;
    case<B>(predicates: {
        some: (value: A) => OptionalAsync<B>;
        none: () => OptionalAsync<B>;
    }): OptionalAsync<B>;
}
export declare const OptAsync: typeof OptionalAsync;
