import { Monad } from "./fantasy";
import { Try } from "./try";
export declare class TryAsync<A> implements Monad<A> {
    static of<B>(value: B | Promise<B>): TryAsync<B>;
    static success<B>(value: B): TryAsync<B>;
    static failure<B>(error: Error): TryAsync<B>;
    static wrap<B>(pTryB: Promise<Try<B>>): TryAsync<B>;
    /**
     * returns an array that keeps only the successful values of an array of TryAsync
     * ```
     * const arr = [Promise.resolve(1), Promise.reject("bad")].map(TryAsync.of);
     * await TryAsync.flatten(arr) // => [1]
     * ```
     * @param {Try<B>[]} arr
     */
    static flatten<B>(arr: TryAsync<B>[]): Promise<B[]>;
    /**
     * Takes an array of TryAsync<B> and returns a single TryAsync containing all the inner values as B[]
     * if they are all successful.
     *
     * @example TryAsync.all([TryAsync.success(1), TryAsync.success(2), TryAsync.success(3)])     => TryAsync.success([1, 2, 3])
     * @example TryAsync.all([TryAsync.success(1), TryAsync.failure(error), TryAsync.success(3)]  => TryAsync.failure(error)
     * @param {TryAsync<B>[]>} arr - array of TryAsync
     */
    static all<B>(arr: TryAsync<B>[]): TryAsync<B[]>;
    /**
     * utility for TryAsync.success<void>(undefined)
     * ```
     * TryAsync.void();
     * ```
     * @return {TryAsync<void>}
     */
    static void(): TryAsync<void>;
    private readonly value;
    private constructor();
    ap<B>(b: TryAsync<(a: A) => B>): TryAsync<B>;
    map<B>(fn: (value: A) => B): TryAsync<B>;
    chain<B>(fn: (value: A) => TryAsync<B>): TryAsync<B>;
    flatMap<B>(fn: (value: A) => TryAsync<B>): TryAsync<B>;
    mapAsync<B>(fn: (value: A) => Promise<B>): TryAsync<B>;
    recover(fn: (error: Error) => A): TryAsync<A>;
    recoverWith(fn: (error: Error) => TryAsync<A>): TryAsync<A>;
    case<B>(predicates: {
        success: (value: A) => TryAsync<B>;
        failure: (error: Error) => TryAsync<B>;
    }): TryAsync<B>;
    promise(): Promise<Try<A>>;
    /**
     * Maps the resolved value to `void` (undefined in this case). In other words, basically just throws away the
     * resolved value. If this is a {@link Failure}, then it will continue to pass along the failure.
     */
    void(): TryAsync<void>;
    get(): Promise<A>;
}
